% EllipsesIntersection 
%
% Computes points and area of intersection between
% two ellipses.
%
% Usage:   [points,area]=EllipsesIntersection(e1,e2)
%
% Arguments:
%          e1,
%          e2 - the first and the second ellipse parameters. 
%               The parameters format is [u,v,a,b,c] as in
%               a(x-u)(x-u)+2b(x-u)(y-v)+c(y-v)(y-v)=1
% Returns:
%          points - a nx2 array of x and y coordinates of points of
%                   intersection. n is between 0 and 4.
%          area - area of intersection. It has value -1 if there is no
%                   intersection.
%
% Author:
% Duan Omerevi
% Faculty of computer and information science
% University of Ljubljana
% http://vicos.fri.uni-lj.si/dusano/
%
% November 2005 - version 1.0
%
% Math sources:
% http://mathforum.org/dr.math/faq/faq.cubic.equations.html
% http://mathworld.wolfram.com/QuadraticEquation.html
% http://mathworld.wolfram.com/CircularSegment.html
% http://www.math.niu.edu/~rusin/known-math/95/ellipse.int
%
% Routine uses cubic equation solver by Herman Bruyninckx.
% See cubic.m for further details.

function [points,area]=EllipsesIntersection(e1,e2)
    % Center of a coordinate system is moved to the center 
    % of the first ellipse 
    t_e2(1:2)=e2(1:2)-e1(1:2);
    t_e1(1:2)=e1(1:2)-e1(1:2);
    
    % Out of arguments we construct a matrix of coefficients
    M1=[e1(3) e1(4);e1(4) e1(5)];
    M2=[e2(3) e2(4);e2(4) e2(5)];

    % Eigenvectors and eigenvalues of the ellipses
    [vec1 val1]=eig(M1);
    [vec2 val2]=eig(M2);
    alpha1=atan2(vec1(4),vec1(3));
    
    % The second ellipse is rotated around the coordinate 
    % system center so that the first ellipse's semimajor
    % and semiminor axis are parallel to the coordinate 
    % system axes
    beta=alpha1-pi/2;
    R1=[cos(beta) sin(beta); -sin(beta) cos(beta)];  
    t2_e2=R1*t_e2(1:2)';    
    r_vec2=R1*vec2;
    MR2=r_vec2*val2*r_vec2';
        
    % The first ellipse is scaled to the unit circle and
    % the second ellipse is scaled accordingly
    t3_e2=sqrt(val1)*t2_e2;
    f=1/sqrt(val1(1)*val1(4));
    MR2_2=[MR2(1)/val1(1) MR2(3)*f;MR2(2)*f MR2(4)/val1(4)];

    % The second ellipse is rotated so that its axes are 
    % parallel to the coordinate system axes
    [vec3 val3]=eig(MR2_2);
    alpha2_2=atan2(vec3(4),vec3(3));
    gama=alpha2_2-pi/2;
    R3=[cos(gama) sin(gama); -sin(gama) cos(gama)];  
    MR2_3=R3*MR2_2*R3';
    t4_e2=R3*t3_e2;;
    
    % We calculate points of intersection between the second ellipse
    % and the unit circle
    points=unitCircleEllipseIntersection(MR2_3(1),MR2_3(4),t4_e2(1),t4_e2(2));
    % We calculate area of intersection between the second ellipse
    % and the unit circle
    area=unitCircleEllipseAreaOfIntersection(MR2_3(1),MR2_3(4),t4_e2(1),t4_e2(2),points);
    
    if area~=-1
        % We denormalize the area of intersection by multiplying it with the
        % first ellipse semimajor and semiminor axis.
        area=area/sqrt(val1(1)*val1(4));
    end
    
    % We denormalize the points of intersection
    if size(points,1)>0
        for i=1:size(points,1)
            point=R3'*points(i,:)';
            point(1)=point(1)/sqrt(val1(1));
            point(2)=point(2)/sqrt(val1(4));
            point=R1'*point;
            points(i,1)=point(1)+e1(1);
            points(i,2)=point(2)+e1(2);
        end
    end
end