% unitCircleEllipseAreaOfIntersection 
%
% Computes points of intersection between
% an unit circle and an ellipse oriented along the coordinate axes.
%
% Usage:   points=unitCircleEllipseIntersection(A,C,u,v)
%
% Arguments:
%          A - 1/sqrt(a) where a is major axis along x coordinate
%          C - 1/sqrt(b) where b is major axis along y coordinate
%          u - x position of the second ellipse center
%          v - y position of the second ellipse center
%          points - a nx2 array of x and y coordinates of points of
%                   intersection. n is between 0 and 4.
% Returns:
%          area - area of intersection
%
% Author:
% Duan Omerevi
% Faculty of computer and information science
% University of Ljubljana
% http://vicos.fri.uni-lj.si/dusano/
%
% November 2005 - version 1.0
%
% Math sources:
% http://mathforum.org/dr.math/faq/faq.cubic.equations.html
% http://mathworld.wolfram.com/QuadraticEquation.html
% http://mathworld.wolfram.com/CircularSegment.html
% http://www.math.niu.edu/~rusin/known-math/95/ellipse.int
%
% Routine uses cubic equation solver by Herman Bruyninckx.
% See cubic.m for further details.

function area=unitCircleEllipseAreaOfIntersection(A,C,u,v,points)

    % Constant specifying when the value is considered zero
    VALUE_EPS=1.0e-05;
    
    area=-1;
    
    if (A<0) | (C<0)
        error('A or C are negative.');
    end
    
    if (A<VALUE_EPS) | (C<VALUE_EPS)
        % Area is zero
        area=0;
        return
    end
    
    points_size=size(points,1);
    if mod(points_size,2)~=0
        error('There must be an even number of points.');
    end
    
    if points_size==0
        % The unit circle and the ellipse do not intersect. This lead to
        % two special cases.
        if (u^2+v^2)<1
            % The ellipse center is within the unit circle. The area of
            % intersection equals the area of the ellipse
            area=min(pi,pi/sqrt(A*C));
        elseif (A*u^2+C*v^2)<1
            % The unit circle center is withing the ellipse. The area of
            % intersection equals the area of the unit circle
            area=pi;
        end
    else
        % Calculation of points angles.
        points(:,3:4)=0;
        for i=1:points_size 
            % The ellipse is transformed to an unit circle too.
            points(i,3)=atan2(sqrt(C)*(points(i,2)-v),sqrt(A)*(points(i,1)-u));
            if points(i,3)<0
                points(i,3)=2*pi+points(i,3);
            end
            points(i,4)=atan2(points(i,2),points(i,1));
            if points(i,4)<0
                points(i,4)=2*pi+points(i,4);
            end
        end
        
        % Points are sorted by an angle
        points=sortrows(points,[3]);
        % Initial value. Just for the case of two double zeros.
        area=pi;
        for i=1:points_size
            if i==points_size
                next_idx=1;
            else
                next_idx=i+1;
            end
            if points(i,3)~=points(next_idx,3)
                % The consecutive points are not double zeros.
                mid_angle=mod(points(i,3)+subtractAngles(points(next_idx,3),points(i,3))/2,2*pi);
                x_mid=cos(mid_angle)/sqrt(A)+u;
                y_mid=sin(mid_angle)/sqrt(C)+v;
                
                if sqrt(x_mid^2+y_mid^2)<1
                    % The chord linking two consecutive points on the
                    % ellipse is within the unit circle
                    if points_size==2
                        % The case of two points intersection
                        
                        % Area between the chord and the ellipse
                        theta=subtractAngles(points(next_idx,3),points(i,3));
                        area=areaOfSector(theta)/sqrt(A*C);
                        
                        % Area between the chord and the unit circle
                        theta=subtractAngles(points(next_idx,4),points(i,4));
                        mid_angle=mod(points(i,4)+theta/2,2*pi);
                        x_mid=cos(mid_angle);
                        y_mid=sin(mid_angle);
                        if sqrt(A*(x_mid-u)^2+C*(y_mid-v)^2)>1
                            % Selected part of the unit circle is outside
                            % of the ellipse. We choose the other part.
                            theta=2*pi-theta;
                        end
                        % Area of intersection is a sum of the area between the
                        % chord and the ellipse and the area  between the
                        % chord and the unit circle
                        area=area+areaOfSector(theta);
                    elseif points_size==4
                        orig_i=i;
                        % In case of four points intersection is the next
                        % segment outside of the unit circle, so we skip
                        % these two points.
                        next_idx=next_idx+2;
                        if next_idx>points_size
                            next_idx=next_idx-points_size;
                        end
                        
                        theta=subtractAngles(points(next_idx,3),points(i,3));
                        if theta==0
                            % Double zero.
                            area=pi/sqrt(A*C);
                        else
                            % Area between the chord and the ellipse
                            area=areaOfSector(theta)/sqrt(A*C);
                            % Area between the chord and the unit circle
                            theta=subtractAngles(points(i,4),points(next_idx,4));
                            % Area of intersection is a sum of the area between the
                            % chord and the ellipse and the area  between the
                            % chord and the unit circle
                            area=area+theta/2-cos(theta/2)*sin(theta/2);
                        end
                        
                        % Now we subtract the next sector which lie outside
                        % of the unit circle.
                        neg_area=0;
                        i=i+1;
                        if i>points_size
                            i=i-points_size;
                        end
                        next_idx=i+1;
                        if next_idx>points_size
                            next_idx=next_idx-points_size;
                        end
                        if points(i,3)~=points(next_idx,3)
                            % Area between the chord and the ellipse
                            theta=subtractAngles(points(next_idx,3),points(i,3));
                            neg_area=areaOfSector(theta)/sqrt(A*C);
                            % Area between the chord and the unit circle
                            theta=subtractAngles(points(next_idx,4),points(i,4));
                            % Area of intersection is a sum of the area between the
                            % chord and the ellipse and the area  between the
                            % chord and the unit circle
                            neg_area=neg_area-(theta/2-cos(theta/2)*sin(theta/2));
                        end
                        area=area-neg_area;
                    else
                        error('Wrong number of points of intersection.');
                    end
                    
                    break;
                end
            end
        end
    end
end

function theta=subtractAngles(alpha,beta)
    if alpha<beta
        theta=2*pi+alpha-beta;
    else
        theta=alpha-beta;
    end
end

function area=areaOfSector(theta)
    if theta>=pi
        theta=2*pi-theta;
        area=pi-(theta/2-cos(theta/2)*sin(theta/2));
    else
        area=theta/2-cos(theta/2)*sin(theta/2);
    end
end