% unitCircleEllipseIntersection 
%
% Computes points of intersection between
% an unit circle and an ellipse oriented along the coordinate axes.
%
% Usage:   points=unitCircleEllipseIntersection(A,C,u,v)
%
% Arguments:
%          A - 1/sqrt(a) where a is major axis along x coordinate
%          C - 1/sqrt(b) where b is major axis along y coordinate
%          u - x position of the second ellipse center
%          v - y position of the second ellipse center
%
% Returns:
%          points - a nx2 array of x and y coordinates of points of
%                   intersection. n is between 0 and 4.
%
% Author:
% Duan Omerevi
% Faculty of computer and information science
% University of Ljubljana
% http://vicos.fri.uni-lj.si/dusano/
%
% November 2005 - version 1.0
%
% Math sources:
% http://mathforum.org/dr.math/faq/faq.cubic.equations.html
% http://mathworld.wolfram.com/QuadraticEquation.html
% http://mathworld.wolfram.com/CircularSegment.html
% http://www.math.niu.edu/~rusin/known-math/95/ellipse.int
%
% Routine uses cubic equation solver by Herman Bruyninckx.
% See cubic.m for further details.

function points=unitCircleEllipseIntersection(A,C,u,v)

    % Constant specifying when the imaginary part is considered zero
    IMAG_EPS=1.0e-05;
    % Constant specifying when the value is considered zero
    VALUE_EPS=1.0e-05;
    % Constant specifying when the u and/or v are considered zero
    UV_EPS=1.0e-02;
    
    if (A<0) | (C<0)
        error('A or C are negative.');
    end
    
    if (A<VALUE_EPS) || (C<VALUE_EPS)
        % The ellipse is degenerate.
        points=[];
        return
    end

    % Intermediate calculations
    R=A*u;
    Q=C*v;
    M=A-C;
    N=-1+C+A*u^2+C*v^2;
    MN=M*N;

    if abs(M)<VALUE_EPS
        % If the ellipse has A==C than the ellipse is a circle. This
        % requires a special case.
        D=-N^2*Q^2+4*Q^4+4*Q^2*R^2;
        F=2*(Q^2+R^2);
        if abs(F)<VALUE_EPS
            % The second ellipse/circle is at the origin. Either all or no
            % points intersect.
            points=[];
            return;
        end
        
        x(1)=(-sqrt(D)+N*R)/F;
        x(2)=(sqrt(D)+N*R)/F;
        
        % Intermediate calculations
        yD=C-A*C*u^2+2*A*C*u*x-A*C*x.^2;
    elseif (abs(u)<UV_EPS) & (abs(v)<UV_EPS)
        % Special case when u==0 and v==0)
        x(1)=-sqrt(1-C)/sqrt(A-C);
        x(2)=x(1);
        x(3)=-x(1);
        x(4)=x(3);
        
        % Intermediate calculations
        yD=C-A*C*x.^2;
    elseif abs(v)<UV_EPS
        % Special case when v==0)
        D=A-C-A*C+C^2+A*C*u^2;
        x(1)=(A*u-sqrt(D))/(A-C);
        x(2)=x(1);
        x(3)=(A*u+sqrt(D))/(A-C);
        x(4)=x(3);
        
        % Intermediate calculations
        yD=C-A*C*u^2+2*A*C*u*x-A*C*x.^2;
    elseif abs(u)<UV_EPS
        % Special case when u==0)
        N=-1+C+C*v^2;
        D=M*(M+N)+Q^2;

        x(1)=sqrt(-2*MN-4*sqrt(D)*Q-4*Q^2)/(sqrt(2)*M);
        x(2)=-x(1);
        x(3)=sqrt(-2*MN+4*sqrt(D)*Q-4*Q^2)/(sqrt(2)*M);
        x(4)=-x(3);
        
        % Intermediate calculations
        yD=C-A*C*x.^2;
    else       
        if abs(M)>1
            e=2*M*N+4*Q^2-2*R^2;
            f=8*Q^2*R;
            g=M^2*(N^2-4*Q^2)-2*M*N*R^2+4*Q^2*R^2+R^4;

            cubic_roots=cubic(1,2*e,e^2-4*g,-f^2);

            s=-1;
            if size(cubic_roots,1)>0
                for i=1:size(cubic_roots,1)
                    if (abs(imag(cubic_roots(i)))<IMAG_EPS) & (cubic_roots(i)>=0)
                        s=real(cubic_roots(i));
                        break;
                    end
                end
            end

            if s>=0
                h=sqrt(s);
                W1=sqrt(s-2*(e-f/h+s));
                W2=sqrt(s-(8*g)/(e-f/h+s));

                x(1)=((-h+W1)/2+R)/M;
                x(2)=((-h-W1)/2+R)/M;;
                x(3)=((h+W2)/2+R)/M;
                x(4)=((h-W2)/2+R)/M;

                % Intermediate calculations
                yD=C-A*C*u^2+2*A*C*u*x-A*C*x.^2;
            else
                x=[];
                yD=[];
            end
        else
            e=(2*N)/M+(4*Q^2-2*R^2)/M^2;
            f=(8*Q^2*R)/M^3;
            g=(N^2-4*Q^2)/M^2-(2*N*R^2)/M^3+(4*Q^2*R^2+R^4)/M^4;

            cubic_roots=cubic(1,2*e,e^2-4*g,-f^2);

            s=-1;
            if size(cubic_roots,1)>0
                for i=1:size(cubic_roots,1)
                    if (abs(imag(cubic_roots(i)))<IMAG_EPS) & (cubic_roots(i)>=0)
                        s=real(cubic_roots(i));
                        break;
                    end
                end
            end

            if s>=0
                h=sqrt(s);
                W1=sqrt(s-2*(e-f/h+s));
                W2=sqrt(s-(8*g)/(e-f/h+s));

                x(1)=(-h+W1)/2+R/M;
                x(2)=(-h-W1)/2+R/M;;
                x(3)=(h+W2)/2+R/M;
                x(4)=(h-W2)/2+R/M;

                % Intermediate calculations
                yD=C-A*C*u^2+2*A*C*u*x-A*C*x.^2;
            else
                x=[];
                yD=[];
            end
        end
    end     

    % Calculation of both possible y coordinates for all points.for
    y=(C*v-sqrt(yD))/C;
    y2=(C*v+sqrt(yD))/C;
    
    % Distance between the calculated points and unit circle. If close to
    % zero than we found an intersection between unit circle and the
    % ellipse
    e1=((x).^2+(y).^2-1).^2;
    e2=((x).^2+(y2).^2-1).^2;
    
    points_idx=0;
    points=zeros(4,2);
    i_size=size(x,2);    
    for i=1:i_size
        % Point is valid only if its imaginary part is considered zero.
        if (abs(imag(x(i)))<IMAG_EPS) & (abs(imag(y(i)))<IMAG_EPS)
            x_value=real(x(i));

            if ((e1(i)<=e2(i)) & (y(i)~=inf)) | (y2(i)==inf)
                y_value=real(y(i));
            else
                y_value=real(y2(i));
            end
            
            if (i<4)
                for j=i+1:i_size
                    if x_value==x(j)
                        if (y_value==y(j)) & (e2(j)<1/sqrt(C))
                            y(j)=inf;
                        elseif y_value==y2(j) & (e1(j)<1/sqrt(C))
                            y2(j)=inf;
                        end
                        break;
                    end
                end
            end
           
            points_idx=points_idx+1;
            points(points_idx,1)=x_value;
            points(points_idx,2)=y_value;
        end
    end
    
    if points_idx==0
        points=[];
    else
        points=points(1:points_idx,:);
    end
end